(* CANAuth, corrected version.
 * To use the abstraction properly, we construct the message as a function of
 * the counter that is being sent out. Note that this is sound since the
 * protocol uses a fresh counter for every message.
 *)

type Channel.
type Cnt.
type Key.

fun msg(Cnt) [private].
type Msg = msg(Cnt).

fun hmac(Key, t).

new ch : Channel.
new sk : Key.

reduc checksign(sk, hmac(sk, m)) = m.
reduc getCnt(msg(cnt)) = cnt.

set sent : Msg.
set seen : Cnt.
set used : Msg.
set used_twice : Msg.

query x:Cnt; att(msg(x)) where (not msg(x) in sent) and (msg(x) in used).
query x:Cnt; att(msg(x)) where (msg(x) in used_twice).

let M(sk) =
    new cnt : Cnt;
    (* Produce the message as a function of the counter *)
    let msg : msg(Cnt) = msg(cnt) in
    update (msg in sent);
    out(ch, (msg, hmac(sk, msg))).

let P(sk) =
    in(ch, (x_msg : Msg, x_sig : hmac(Key, Msg)));
    let =x_msg = checksign(sk, x_sig) in
    (* Retrieve the counter part of the message. *)
    let x_cnt:Cnt = getCnt(x_msg) in
    if not x_cnt in seen then
       if x_msg in used
       then update (x_cnt in seen and x_msg in used_twice)
       else update (x_cnt in seen and x_msg in used).

process (!{sent}M(sk)) | (!{seen, used, used_twice}P(sk))
